﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using Newtonsoft.Json;

namespace MontoyaTech.Rest.Net
{
    /// <summary>
    /// A set of extensions to help with HttpListenerRequests.
    /// </summary>
    public static class HttpListenerRequestExtensions
    {
        /// <summary>
        /// Reads the content of a HttpListenerRequest as a string and returns it.
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public static string ReadAsString(this HttpListenerRequest request)
        {
            try
            {
                using (var input = request.InputStream)
                    using (var stream = new StreamReader(input))
                        return stream.ReadToEnd();
            }
            catch
            {
                return "";
            }
        }

        /// <summary>
        /// Reads the content of a HttpListenerRequest as a json object and returns it.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="request"></param>
        /// <returns></returns>
        public static T ReadAsJson<T>(this HttpListenerRequest request)
        {
            try
            {
                using (var input = request.InputStream)
                    using (var stream = new StreamReader(input))
                        return JsonConvert.DeserializeObject<T>(stream.ReadToEnd());
            }
            catch
            {
                return default(T);
            }
        }

        /// <summary>
        /// Reads the content of a HttpListenerRequest as a byte array.
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public static byte[] ReadAsBytes(this HttpListenerRequest request)
        {
            try
            {
                using (var input = request.InputStream)
                {
                    using (var stream = new MemoryStream())
                    {
                        input.CopyTo(stream);
                        return stream.ToArray();
                    }
                }
            }
            catch
            {
                return null;
            }
        }
    }
}
